import numpy as np
import cv2
from skimage.morphology import label

def rle_decode(mask_rle, shape, color=1):
    '''
    mask_rle: run-length as string formated (start length)
    shape: (height, width, channels) of array to return 
    color: color for the mask
    Returns numpy array (mask)

    '''
    s = mask_rle.split()
    
    starts = list(map(lambda x: int(x) - 1, s[0::2]))
    lengths = list(map(int, s[1::2]))
    ends = [x + y for x, y in zip(starts, lengths)]
    
    img = np.zeros((shape[0] * shape[1], shape[2]), dtype=np.float32)
            
    for start, end in zip(starts, ends):
        img[start : end] = color
    
    return img.reshape(shape)


'''
# Run-length encoding - https://www.kaggle.com/rakhlin/fast-run-length-encoding-python
'''
def rle_encoding(x):

    dots = np.where(x.T.flatten() == 1)[0]
    run_lengths = []
    prev = -2
    for b in dots:
        if (b>prev+1): run_lengths.extend((b + 1, 0))
        run_lengths[-1] += 1
        prev = b
    return run_lengths

def prob_to_rles(x, cutoff=0.5):
    lab_img = label(x > cutoff)
    for i in range(1, lab_img.max() + 1):
        yield rle_encoding(lab_img == i)

def plot_masks(image_id, labels, path, axs, colors=True):
    if colors:
        mask = np.zeros((520, 704, 3))
        for label in labels:
            mask += rle_decode(label, shape=(520, 704, 3), color=np.random.rand(3))
    else:
        mask = np.zeros((520, 704, 1))
        for label in labels:
            mask += rle_decode(label, shape=(520, 704, 1))
    mask = mask.clip(0, 1)

    image = cv2.imread(path + f"{image_id}.png")
    image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

    axs[0][0].imshow(image)
    axs[0][0].set_title('Input image')

    axs[0][1].imshow(image)
    axs[0][1].imshow(mask[:,:,0], alpha=0.5)
    axs[0][1].set_title('Input image with mask')

    axs[1][0].imshow(mask[:,:,0])
    axs[1][0].set_title('Only mask')

    for ax in axs.ravel(): ax.axis("off")